<?php
/* wppa-cache.php
/*
/* Contains all wppa smart cache functions
/*
/* Version 8.0.08.004
*/

// Test for caching
//
// Used for shortcodes.
// Returns array( 'caching' 	=> BOOL, indicating if current shortcode will be cached
//				  'cache_id' 	=> STRING, indicates the page and the occur, the login/out/admin, language  etc
//				  'cache_file'	=> STRING, pathname to the cache file
//				  'cache_data' 	=> STRING, content of the cachefile
//				 );
function wppa_test_for_caching() {
global $wppa_lang;

	// Assume not
	$caching 	= false;
	$cache_id 	= false;
	$cache_file = false;
	$cache_data = false;

	if ( wppa_get( 'cache' ) ) {
		wppa( 'cache', wppa_get( 'cache' ) );
	}

	if ( wppa( 'cache' ) ) {

		// No querystring?
		if ( ! wppa_in_widget() && ! $_SERVER['REQUEST_URI'] ) {
			$caching = true;
		}

		// Querystring not for this occur?
		elseif ( ! wppa_in_widget() && wppa( 'mocc' ) != wppa_get( 'occur' ) ) {
			$caching = true;
		}

		// Ajax?
		elseif ( wppa_get( 'cache' ) ) {
			$caching = true;
		}

		if ( $caching ) {

			$cache_id = str_replace( '/', '.', get_permalink() ) . '-' . wppa( 'mocc' );
			if ( wppa( 'ajax' ) ) {
				$cache_id .= $_SERVER['QUERY_STRING'];
			}
			$cache_id = str_replace( array( '..', '.', '_' ), '-', $cache_id );
			$cache_id = str_replace( '--', '-', $cache_id );
			$cache_id = sanitize_file_name( $cache_id );
			$root = WPPA_CONTENT_PATH . '/' . wppa_opt( 'cache_root' ) .'/wppa-shortcode';
			if ( ! wppa_is_dir ( $root ) ) {
				wppa_mktree( $root );
			}
			$cache_file = $root . '/' . $wppa_lang . '-' . ( is_user_logged_in() ? 'login-' : '' ) . ( wppa_user_is( 'administrator' ) ? 'admin-' : '' ) . ( wppa_is_mobile() ? 'mobile-' : '' ) . $cache_id;

			// Cachefile present?
			if ( wppa_is_file( $cache_file ) ) {
				$timer = wppa_occur_timer( 'show', $_SERVER['REQUEST_URI'] . ' oc ' . wppa( 'mocc' ), true );
				wppa_log( 'fso', 'Reading cachefile ' . str_replace( WPPA_CONTENT_PATH, '...', $cache_file ) );

				$cache_data = wppa_get_contents( $cache_file ) . $timer;
				update_option( 'wppa_cache_hits', get_option( 'wppa_cache_hits', 0 ) +1 );
			}
		}
	}

	$result = array( 'caching' 		=> $caching,
					 'cache_id' 	=> $cache_id,
					 'cache_file'	=> $cache_file,
					 'cache_data' 	=> $cache_data,
					 );

	return $result;
}

// Save cache file
//
// Args array( 'file' 		=> STRING, pathname to cachefile
// 	 	 	   'data' 		=> STRING, content of the cachefile
// 			   'albums' 	=> STRING, enumeration of album ids, OR '*' indicating all albums,
// 			   'photos'		=> STRING, enumeration of photo ids, OR '*' indicating all photos,
// 			   'other' 		=> CHAR, 'C' for comment, 'R' for rating, default ''
//			 );
// to indicate on what changes the cachefile should be purged. This info (cache metadata) is stored in $wpdb->wppa_caches
function wppa_save_cache_file( $xargs ) {
global $wppa_current_shortcode;
global $albums_used;
global $photos_used;
global $other_deps;

	// Fill in defaults
	$defaults = array( 'file'	=> '',
					   'data' 	=> '',
					   'albums' => $albums_used,
					   'photos' => $photos_used,
					   'other' 	=> $other_deps,
					   'remark' => $wppa_current_shortcode,
					   );
	$args = wp_parse_args( (array) $xargs, $defaults );

	// Extract vars
	$file 	= $args['file'];
	$data 	= $args['data'];
	$albums = $args['albums'];
	$photos = $args['photos'];
	$other 	= $args['other'];
	$remark = $args['remark'];

	// Check for missing refs
	if ( ! $albums && ! $photos && ! $other ) {
		wppa_log( 'err', 'Missing album/photo/other ref for cachefile ' . str_replace( WPPA_CONTENT_PATH, '...', $file ) );
	}

	// Sanitize input
	if ( $albums != '*' ) {

		if ( ! $albums || substr( $albums, 0, 1 ) == '$' ) {
			$albums = wppa( 'start_album' );
		}

		// Remove dups, remove virtual album indicators
		$albums = explode( '.', $albums );
		foreach( array_keys( $albums ) as $key ) {
			if ( substr( $albums[$key], 0, 1 ) == '#' ) {
				unset( $albums[$key] );
			}
		}
		$albums = array_unique( $albums, SORT_NUMERIC );
		$albums = implode( '.', $albums );
		$albums = trim( $albums, '.' );
	}

	if ( $photos != '*' ) {

		if ( ! $photos ) {
			$photos = wppa( 'start_photo' ) . '.' . wppa_expand_enum( wppa( 'start_photos' ) );
		}

		// Remove dups
		$photos = explode( '.', $photos );
		$photos = array_unique( $photos, SORT_NUMERIC );
		$photos = implode( '.', $photos );
		$photos = trim( $photos, '.' );
	}

	if ( $other != 'C' && $other != 'R' ) {
		$other = '';
	}

	// Log making cachefile
	$extra = '';
	if ( $remark ) {
		$extra = ' for {b}' . $remark . '{/b}. ';
	}
	wppa_log( 'fso', 'Writring cachefile ' . str_replace( WPPA_CONTENT_PATH, '...', $file ) . $extra );

	// Save it
	wppa_put_contents( $file, $data );

	// Register cache miss
	update_option( 'wppa_cache_misses', get_option( 'wppa_cache_misses', 0 ) +1 );

	// Save used albums and photos
	wppa_create_cache_entry( array( 'filename' 	=> str_replace( WPPA_CONTENT_PATH, '...', $file ),
									'albums' 	=> $albums,
									'photos' 	=> $photos,
									'other' 	=> $other,
									) );
	// Cleanup
	$albums_used = '';
	$photos_used = '';
	$other_deps  = '';
}

// Clear cache
//
// Args array( 'album' 		=> INT album id, will delete all caches that contain indicated album
//			   'photo' 		=> INT photo id, will delete all caches that contain indicated photo
//			   'force' 		=> BOOL if true, will delete all caches
//			   'shortcodes' => BOOL if true, will delete all shortcode caches
//			   'widgets' 	=> BOOL if true, will delete all widget caches
//			   'albums' 	=> BOOL if true, will delete all caches that contain any album
//			   'photos' 	=> BOOL if true, will delete all caches that contain any photo
//			   'qr' 		=> BOOL if true, will delete all qr code caches
// 			   'other' 		=> CHAR either '', 'C', or 'R', will delete all caches that contain Comments or Ratings
//			 );
//
function wppa_clear_cache( $args = array() ) {
global $cache_path;
global $wpdb;

	wppa_log( 'obs', 'Clear cache called with args ' . var_export( $args, true ) );

	// Fill in default args
	$defaults = array( 	'album' 		=> '',
						'photo' 		=> '',
						'force' 		=> false,
						'shortcodes' 	=> false,
						'widgets' 		=> false,
						'albums' 		=> false,
						'photos' 		=> false,
						'qr' 			=> false,
						'other' 		=> '',
						);
	$args = wp_parse_args( (array) $args, $defaults );

	$album 		= $args['album'];
	$photo 		= $args['photo'];
	$force 		= $args['force'];
	$shortcodes = $args['shortcodes'] || $force;
	$widgets 	= $args['widgets'] || $force;
	$albums 	= $args['albums'] || $force;
	$photos 	= $args['photos'] || $force;
	$qr 		= $args['qr'] || $force;
	$other 		= $args['other'];

	// Album based
	if ( $album != '' ) {

		$query = "SELECT filename FROM $wpdb->wppa_caches WHERE albums LIKE '%.$album.%' OR albums = '*'";
		$files = $wpdb->get_col( $query );
		foreach( $files as $file ) {
			$path = str_replace( '...', WPPA_CONTENT_PATH, $file );
			if ( wppa_is_file( $path ) ) {
				wppa_unlink( $path, true );
			}
		}
		$wpdb->query( "DELETE FROM $wpdb->wppa_caches WHERE albums LIKE '%.$album.%' OR albums = '*'" );
	}

	// Photo based
	if ( $photo != '' ) {

		$query = "SELECT filename FROM $wpdb->wppa_caches WHERE photos LIKE '%.$photo.%' OR photos = '*'";
		$files = $wpdb->get_col( $query );
		foreach( $files as $file ) {
			$path = str_replace( '...', WPPA_CONTENT_PATH, $file );
			if ( wppa_is_file( $path ) ) {
				wppa_unlink( $path, true );
			}
		}
		$wpdb->query( "DELETE FROM $wpdb->wppa_caches WHERE photos LIKE '%.$photo.%' OR photos = '*'" );
	}

	// All albums
	if ( $albums ) {

		$query = "SELECT filename FROM $wpdb->wppa_caches WHERE albums <> '..'";
		$files = $wpdb->get_col( $query );
		foreach( $files as $file ) {
			$path = str_replace( '...', WPPA_CONTENT_PATH, $file );
			if ( wppa_is_file( $path ) ) {
				wppa_unlink( $path, true );
			}
		}
		$wpdb->query( "DELETE FROM $wpdb->wppa_caches WHERE albums <> '..'" );
	}

	// All photos
	if ( $photos ) {

		$query = "SELECT filename FROM $wpdb->wppa_caches WHERE photos <> '..'";
		$files = $wpdb->get_col( $query );
		foreach( $files as $file ) {
			$path = str_replace( '...', WPPA_CONTENT_PATH, $file );
			if ( wppa_is_file( $path ) ) {
				wppa_unlink( $path, true );
			}
		}
		$wpdb->query( "DELETE FROM $wpdb->wppa_caches WHERE photos <> '..'" );
	}

	// QR code caches
	if ( $qr ) {
		if ( wppa_is_dir( WPPA_UPLOAD_PATH . '/qr' ) ) {
			$qrs = wppa_glob( WPPA_UPLOAD_PATH . '/qr/*.svg' );
			if ( ! empty( $qrs ) ) {
				foreach( $qrs as $path ) wppa_unlink( $path, true );
			}
		}
	}

	// Clear all shortcode caches
	if ( $shortcodes ) {

		$root = WPPA_CONTENT_PATH . '/' . wppa_opt( 'cache_root' ) . '/wppa-shortcode';
		if ( wppa_is_dir( $root ) ) {
			wppa_tree_empty( $root );
		}
		$wpdb->query( "DELETE FROM $wpdb->wppa_caches WHERE filename LIKE '%/wppa-shortcode/%'" );
		wppa_log( 'fso', 'All wppa shortcode caches cleared' );
	}

	// Clear all widget caches
	if ( $widgets ) {

		$root = WPPA_CONTENT_PATH . '/' . wppa_opt( 'cache_root' ) . '/wppa-widget';
		if ( wppa_is_dir( $root ) ) {
			wppa_tree_empty( $root );
		}
		$wpdb->query( "DELETE FROM $wpdb->wppa_caches WHERE filename LIKE '%/wppa-widget/%'" );
		wppa_log( 'fso', 'All wppa widget caches cleared' );
	}

	// Clear Other caches
	if ( $other ) {

		$query = "SELECT filename FROM $wpdb->wppa_caches WHERE other = '$other'";
		$files = $wpdb->get_col( $query );
		foreach( $files as $file ) {
			$path = str_replace( '...', WPPA_CONTENT_PATH, $file );
			if ( wppa_is_file( $path ) ) {
				wppa_unlink( $path, true );
			}
		}
		$wpdb->query( "DELETE FROM $wpdb->wppa_caches WHERE other = '$other'" );

	}

	// Cleanup tempfiles and clear non wppa caches
	if ( $force ) {

		// Remove tempfiles
		wppa_delete_obsolete_tempfiles();

		// If wp-super-cache is on board, clear cache
		if ( function_exists( 'prune_super_cache' ) ) {
			prune_super_cache( $cache_path . 'supercache/', true );
			prune_super_cache( $cache_path, true );
			wppa_log('obs', 'Super cache cleared');
		}

		// W3 Total cache
		if ( function_exists( 'w3tc_pgcache_flush' ) ) {
			w3tc_pgcache_flush();
			wppa_log('obs', 'w3tc pgcache cleared');
		}

		// SG_CachePress
		if ( class_exists( 'SG_CachePress_Supercacher' ) ) {
			$c = new SG_CachePress_Supercacher();
			if ( $c->purge_cache ) {
				$c->purge_cache();
				wppa_log('obs', 'SG CachePress Supercacher cache cleared');
			}
		}

		// Quick cache
		if ( isset($GLOBALS['quick_cache']) ) {
			$GLOBALS['quick_cache']->clear_cache();
			wppa_log('obs', 'Quick cache cleared');
		}

		// Comet cache
		if ( class_exists( 'comet_cache' ) ) {
			comet_cache::clear();
			wppa_log('obs', 'Comet cache cleared');
		}

		wppa_log( 'fso', 'All non wppa caches cleared' );
	}
}

// Get path to widget cache file
function wppa_get_widget_cache_path( $widget_id ) {
global $wppa_lang;

	$root = WPPA_CONTENT_PATH . '/' . wppa_opt( 'cache_root' ) . '/wppa-widget';

	if ( ! wppa_is_dir( $root ) ) {
		wppa_mktree( $root );
	}

	return 	$root . '/' . $wppa_lang . '-' . ( is_user_logged_in() ? 'login-' : '' ) . ( wppa_user_is( 'administrator' ) ? 'admin-' : '' ) . $widget_id;
}

// Remove cache files for a certrain widget
function wppa_remove_widget_cache( $widget_id ) {
global $wpdb;

	if ( ! $widget_id ) return;

	$files = $wpdb->get_col( "SELECT filename FROM $wpdb->wppa_caches WHERE filename LIKE '%$widget_id%'" );
	foreach ( $files as $file ) {
		$path = str_replace( '...', WPPA_CONTENT_PATH, $file );
		if ( wppa_is_file( $path ) ) {
			wppa_unlink( $path, true );
		}
	}
	$wpdb->query( "DELETE FROM $wpdb->wppa_caches WHERE filename LIKE '%$widget_id%'" );
}